_use sql;
_COMPARE SET;
_COMPARE SET MASK CASE IGBLANK NOTRIM;
_COMPARE SET;
_COMPARE RESET;
_COMPARE SET;
_COMPARE SKIPLINE "aa*bb";
_COMPARE SKIPLINE "cc*dd";
_COMPARE NOSKIPLINE "aa*bb";
_COMPARE MASKLINE "aa bb"=>"cc bb";
_COMPARE MASKLINE "ee ff"=>"hh kk";
_COMPARE NOMASKLINE "aa bb";
_COMPARE SET;
_COMPARE RESET;
_COMPARE SET OUTPUT CONSOLE DIFFFILE;
_COMPARE testcomparework.txt testcompareref.txt MASK NOCASE;

